@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from fontTools.misc.py23 import *
+from fontTools.misc.py23 import bytechr, byteord, bytesjoin, strjoin, tobytes, tostr
 from fontTools.misc import sstruct
 from fontTools.misc.textTools import safeEval
 from fontTools.misc.encodingTools import getEncoding
@@ -133,7 +133,7 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 		"""
 		if not hasattr(self, 'names'):
 			self.names = []
-		if not isinstance(string, unicode):
+		if not isinstance(string, str):
 			if isinstance(string, bytes):
 				log.warning(
 					"name string is bytes, ensure it's correctly encoded: %r", string)
@@ -184,7 +184,7 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 			raise ValueError("nameID must be less than 32768")
 		return nameID
 
-	def findMultilingualName(self, names, windows=True, mac=True):
+	def findMultilingualName(self, names, windows=True, mac=True, minNameID=0):
 		"""Return the name ID of an existing multilingual name that
 		matches the 'names' dictionary, or None if not found.
 
@@ -198,6 +198,9 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 		platEncID=1.
 		If 'mac' is True, the returned name ID is guaranteed to exist
 		for all requested languages for platformID=1 and platEncID=0.
+
+		The returned name ID will not be less than the 'minNameID'
+		argument.
 		"""
 		# Gather the set of requested
 		#   (string, platformID, platEncID, langID)
@@ -222,9 +225,12 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 		# Collect matching name IDs
 		matchingNames = dict()
 		for name in self.names:
-			key = (name.string, name.platformID,
-			       name.platEncID, name.langID)
-			if key in reqNameSet:
+			try:
+				key = (name.toUnicode(), name.platformID,
+				       name.platEncID, name.langID)
+			except UnicodeDecodeError:
+				continue
+			if key in reqNameSet and name.nameID >= minNameID:
 				nameSet = matchingNames.setdefault(name.nameID, set())
 				nameSet.add(key)
 
@@ -236,7 +242,7 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 		return None  # not found
 
 	def addMultilingualName(self, names, ttFont=None, nameID=None,
-	                        windows=True, mac=True):
+	                        windows=True, mac=True, minNameID=0):
 		"""Add a multilingual name, returning its name ID
 
 		'names' is a dictionary with the name in multiple languages,
@@ -255,12 +261,16 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 
 		If 'windows' is True, a platformID=3 name record will be added.
 		If 'mac' is True, a platformID=1 name record will be added.
+
+		If the 'nameID' argument is None, the created nameID will not
+		be less than the 'minNameID' argument.
 		"""
 		if not hasattr(self, 'names'):
 			self.names = []
 		if nameID is None:
 			# Reuse nameID if possible
-			nameID = self.findMultilingualName(names, windows=windows, mac=mac)
+			nameID = self.findMultilingualName(
+				names, windows=windows, mac=mac, minNameID=minNameID)
 			if nameID is not None:
 				return nameID
 			nameID = self._findUnusedNameID()
@@ -300,10 +310,9 @@ class table__n_a_m_e(DefaultTable.DefaultTable):
 			"'platforms' must contain at least one (platformID, platEncID, langID) tuple"
 		if not hasattr(self, 'names'):
 			self.names = []
-		if not isinstance(string, unicode):
+		if not isinstance(string, str):
 			raise TypeError(
-				"expected %s, found %s: %r" % (
-					unicode.__name__, type(string).__name__,string ))
+				"expected str, found %s: %r" % (type(string).__name__, string))
 		nameID = self._findUnusedNameID(minNameID + 1)
 		for platformID, platEncID, langID in platforms:
 			self.names.append(makeName(string, nameID, platformID, platEncID, langID))
@@ -431,7 +440,7 @@ class NameRecord(object):
 		encoding = self.getEncoding()
 		string = self.string
 
-		if encoding == 'utf_16_be' and len(string) % 2 == 1:
+		if isinstance(string, bytes) and encoding == 'utf_16_be' and len(string) % 2 == 1:
 			# Recover badly encoded UTF-16 strings that have an odd number of bytes:
 			# - If the last byte is zero, drop it.  Otherwise,
 			# - If all the odd bytes are zero and all the even bytes are ASCII,
@@ -447,7 +456,7 @@ class NameRecord(object):
 			elif byteord(string[0]) == 0 and all(isascii(byteord(b)) for b in string[1:]):
 				string = bytesjoin(b'\0'+bytechr(byteord(b)) for b in string[1:])
 
-		string = tounicode(string, encoding=encoding, errors=errors)
+		string = tostr(string, encoding=encoding, errors=errors)
 
 		# If decoded strings still looks like UTF-16BE, it suggests a double-encoding.
 		# Fix it up.
